#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "aduc8xx.h"

void print_help(void){
  printf("\nUsage:");
  printf("\taduc8xx [-p device] -c|-a [-w hex-file] [-e hex-file]\n");
  printf("\n");
  printf("Options:\n");
  printf("\t-w write program memory\n");
  printf("\t-e write data memory\n");
  printf("\t-c erase program memory\n");
  printf("\t-a erase program and data memory\n\n");
}

void *allocateMemory(int bufferSize)
{
  void *p;

  p = malloc(bufferSize);
  if (p == NULL) {
    fprintf(stderr, "Could not allocate buffer memory.\n");
  } else {
    strnset(p, 0, bufferSize);
  }
  return p;
}

int readHexFile(char *fname, int bufferSize, unsigned char *buffer)
{
  FILE *fp;

  printf("Read hexfile - %s.\n", fname);
  fp = fopen(fname, "r");
  if (fp == NULL) {
    fprintf(stderr, "Could not open hexfile - %s.\n", fname);
    return 0;
  }
  if (readFromHexInto(fp, bufferSize, buffer) != 0) {
    printf("Hex read error.\n");
    fclose(fp);
    return 0;
  }
  fclose(fp);
  return 1;
}

main(int argc, char **argv)
{
  int idnum, fd, bufferSize;
  unsigned char *buffer;
  char c, *p;
  char *dev_name = "COM1";
  char *program_hexfile;
  char *data_hexfile;
  FILE *fp;
  int opt_w = 0, opt_e = 0, opt_c = 0, opt_a = 0, opt_v = 0;

/*  while ((c = getopt(argc, argv, "w:e:p:cahv")) != -1) { */
  while (--argc) {
    if (**++argv == '-') {
      p = *argv;
      while ((c = *++p) != '\0') {
        switch(c){
        case 'w':
          opt_w = 1;
          if (--argc < 1) {
            print_help();
            exit(1);
          } 
          program_hexfile = *++argv;
          break;
        case 'e':
          opt_e = 1;
          if (--argc < 1) {
            print_help();
            exit(1);
          }
          data_hexfile = *++argv;
          break;
        case 'p':
          if (--argc < 1) {
            print_help();
            exit(1);
          }
          dev_name = *++argv;
          break;
        case 'c':
          opt_c = 1;
          break;
        case 'a':
          opt_a = 1;
          break;
        case 'v':
          opt_v = 1;
          break;
        case 'h':
          print_help();
          exit(1);
          break;
        default:
          break;
        }
      }
    } else {
      print_help();
      exit(1);
    }
  }

  if (!(opt_w || opt_e || opt_c || opt_a || opt_v)) {
    print_help();
    exit(0);
  }
  
  if ((fd = openSerialPort(dev_name)) < 0) {
    fprintf(stderr, "Could not open %s.\n", dev_name);
    exit(1);
  }

  if (opt_v) {
    printf("aduc8xx ver.0.1\n");
  }

  clearInputBuffer(fd);
  if ((idnum = identify(fd)) < 1) {
    exit(1);
  }

  if (opt_c) {
    eraseProgramMemory(fd);
  }
  if (opt_a) {
    eraseProgramAndDataMemory(fd);
  }
  if (opt_w) {
    bufferSize = pmsize(idnum);
    if ((buffer = allocateMemory(bufferSize)) == NULL) {
      closeSerialPort(fd);
      exit(0);
    }
    if (readHexFile(program_hexfile, bufferSize, buffer) != 1) {
      closeSerialPort(fd);
      exit(0);
    }
    writeProgramMemoryFrom(fd, idnum, buffer);
    free(buffer);
  }
  if (opt_e) {
    bufferSize = dmsize(idnum);
    if ((buffer = allocateMemory(bufferSize)) == NULL) {
      closeSerialPort(fd);
      exit(0);
    }
    if (readHexFile(data_hexfile, bufferSize, buffer) != 1) {
      closeSerialPort(fd);
      exit(0);
    }
    writeDataMemoryFrom(fd, idnum, buffer);
    free(buffer);
  }

  closeSerialPort(fd);
  return 0;
}
